#pragma once

#include <ArmarXCore/core/Component.h>

#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>
#include <RobotAPI/libraries/armem/core/MemoryID.h>


namespace armarx::armem::mns
{
    class ClientPluginUserBase;
}


namespace armarx::armem::mns::plugins
{
    /**
     * @brief A base plugin offering optional access and dependency
     * to the Memory Name System (MNS).
     */
    class ClientPlugin : public ComponentPlugin
    {
    public:

        using ComponentPlugin::ComponentPlugin;
        virtual ~ClientPlugin() override;

        void postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties) override;

        void preOnInitComponent() override;
        void preOnConnectComponent() override;

        /**
         * @brief Indicate whether the Memory Name System (MNS) is enabled.
         */
        bool isMemoryNameSystemEnabled();
        /**
         * @brief Get the name of the MNS component.
         */
        std::string getMemoryNameSystemName();

        /**
         * @brief Get the MNS proxy.
         * @return The MNS proxy when MNS is enabled, nullptr when MNS is disabled.
         */
        mns::MemoryNameSystemInterfacePrx getMemoryNameSystem();

    public:
        static constexpr const char* PROPERTY_MNS_ENABLED_NAME = "mns.MemoryNameSystemEnabled";
        static constexpr const char* PROPERTY_MNS_NAME_NAME = "mns.MemoryNameSystemName";
    };


    /**
     * @brief Base class for users of the `ClientPlugin`.
     * This is itself not a usable plugin user (hence still in the plugins namespace).
     */
    class ClientPluginUserBase
    {
    protected:

        virtual ~ClientPluginUserBase();


        armem::data::WaitForMemoryResult waitForMemory(const std::string& memoryName);
        armem::data::ResolveMemoryNameResult resolveMemoryName(const std::string& memoryName);
        bool isMemoryAvailable(const std::string& memoryName);

        armem::data::WaitForMemoryResult useMemory(const MemoryID& id);
        virtual armem::data::WaitForMemoryResult useMemory(const std::string& memoryName);


    public:

        /// Only set when enabled.
        mns::MemoryNameSystemInterfacePrx memoryNameSystem = nullptr;

        bool memoryNameSystemEnabled = true;
        std::string memoryNameSystemName = "MemoryNameSystem";

    };
}
