/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <Eigen/Geometry>

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

namespace armarx
{

    struct TransformHeader
    {
        std::string parentFrame;
        std::string frame;

        std::string agent;

        std::int64_t timestamp; // [µs]
    };

    struct Transform
    {
        TransformHeader header;

        Eigen::Affine3f transform = Eigen::Affine3f::Identity();
    };

    namespace armem
    {

        struct TransformResult
        {
            Transform transform;

            enum class Status
            {
                Success,
                ErrorLookupIntoFuture,
                ErrorFrameNotAvailable
            } status;

            explicit operator bool() const
            {
                return status == Status::Success;
            }

            std::string errorMessage = "";
        };

        struct TransformQuery
        {
            TransformHeader header;

            // bool exact;
        };

        class TransformInterface
        {
        public:
            virtual ~TransformInterface() = default;

            virtual void registerPropertyDefinitions(PropertyDefinitionsPtr& def) = 0;
            virtual void connect() = 0;
        };

        class TransformReaderInterface : virtual public TransformInterface
        {
        public:
            virtual ~TransformReaderInterface() = default;

            virtual TransformResult
            getGlobalPose(const std::string& agentName, const std::string& robotRootFrame,
                          const std::int64_t& timestamp) const = 0;

            virtual TransformResult
            lookupTransform(const TransformQuery& query) const = 0;
            // waitForTransform()
        };

        class TransformWriterInterface : virtual public TransformInterface
        {
        public:
            ~TransformWriterInterface() override = default;

            virtual bool commitTransform(const Transform& transform) = 0;
        };

    } // namespace armem

} // namespace armarx