/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <stdint.h>
#include <string>
#include <vector>

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/libraries/armem/client/Reader.h>
// TODO(fabian.reister): move MemoryConnector to armem library
#include <RobotAPI/libraries/armem_robot_localization/MemoryConnector.h>
#include <RobotAPI/libraries/armem/client/query/Builder.h>
#include <RobotAPI/libraries/armem_robot_mapping/types.h>



namespace armarx
{
    class ManagedIceObject;
}

namespace armarx::armem
{

    struct TimeRange
    {
        Time min;
        Time max;
    };

    /**
    * @defgroup Component-ExampleClient ExampleClient
    * @ingroup RobotAPI-Components
    * A description of the component ExampleClient.
    *
    * @class ExampleClient
    * @ingroup Component-ExampleClient
    * @brief Brief description of class ExampleClient.
    *
    * Detailed description of class ExampleClient.
    */
    class MappingDataReader :
        virtual public armarx::armem::MemoryConnector
    {
    public:
        MappingDataReader(ManagedIceObject& component);

        virtual ~MappingDataReader();

        void connect();


        struct Query
        {
            std::string agent;

            TimeRange timeRange;

            // if empty, all sensors will be queried
            std::vector<std::string> sensorList;

        };

        struct Result
        {

            std::vector<LaserScanStamped> laserScans;

            std::vector<std::string> sensors;

            enum Status
            {
                Error,
                Success
            } status;

            std::string errorMessage;

        };

        Result queryData(const Query& query) const;

        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);

        const std::string& getPropertyPrefix() const override
        {
            return propertyPrefix;
        }

        client::query::Builder buildQuery(const Query& query) const ;


    private:

        armem::client::Reader memoryReader;

        // Properties
        struct Properties
        {
            std::string memoryName             = "RobotState";
            std::string mappingMemoryName = "Mapping";
        } properties;


        const std::string propertyPrefix = "mem.mapping.read.";
    };

}  // namespace armarx::armem
