/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */


// STD/STL
#include <string>
#include <map>

// Header
#include "Primitive.h"

namespace armarx::aron::cppcodegenerator::serializer
{

    namespace
    {
        std::string convertStdString(const std::string& s)
        {
            if (s == "string")
            {
                return "std::string";
            }
            return s;
        }
    }
#define RUN_ARON_MACRO(upperType, lowerType, capsType, upperData, lowerData, capsData) \
    /* constructors */ \
    upperType##Serializer::upperType##Serializer(const typenavigator::upperType##NavigatorPtr& e) : \
        Serializer(convertStdString(#lowerData), simox::meta::get_type_name(typeid(data::Aron##upperData)), simox::meta::get_type_name(typeid(type::Aron##upperType))), \
        typenavigator(e) \
    { \
        \
    } \
    \
    /* virtual implementations */ \
    std::vector<CppFieldPtr> upperType##Serializer::getPublicVariableDeclarations(const std::string& name) const \
    { \
        CppFieldPtr field = CppFieldPtr(new CppField(getFullCppTypename(), name)); \
        return {field}; \
    } \
    \
    std::vector<std::pair<std::string, std::string>> upperType##Serializer::getCtorInitializers(const std::string&) const \
    { \
        return {}; \
    } \
    \
    CppBlockPtr upperType##Serializer::getCtorBlock(const std::string&) const \
    { \
        CppBlockPtr b = CppBlockPtr(new CppBlock()); \
        return b; \
    } \
    \
    CppBlockPtr upperType##Serializer::getResetBlock(const std::string& accessor) const \
    { \
        CppBlockPtr b = CppBlockPtr(new CppBlock()); \
        b->addLine(accessor + " = {};"); \
        return b; \
    } \
    \
    CppBlockPtr upperType##Serializer::getInitializeBlock(const std::string& accessor) const \
    { \
        CppBlockPtr b = CppBlockPtr(new CppBlock()); \
        b->addLine(accessor + " = {};"); \
        return b; \
    } \
    \
    CppBlockPtr upperType##Serializer::getWriteInitialTypeBlock(const std::string&) const \
    { \
        CppBlockPtr b = CppBlockPtr(new CppBlock()); \
        b->addLine("w.write" + std::string(#upperType) + "();"); \
        return b; \
    } \
    \
    CppBlockPtr upperType##Serializer::getWriteCurrentTypeBlock(const std::string&) const \
    { \
        CppBlockPtr b = CppBlockPtr(new CppBlock()); \
        b->addLine("w.write" + std::string(#upperType) + "();"); \
        return b; \
    } \
    \
    CppBlockPtr upperType##Serializer::getWriteBlock(const std::string& accessor) const \
    { \
        CppBlockPtr b = CppBlockPtr(new CppBlock()); \
        b->addLine("w.writePrimitive(" + accessor + ");"); \
        return b; \
    } \
    \
    CppBlockPtr upperType##Serializer::getReadBlock(const std::string& accessor) const \
    { \
        CppBlockPtr b = CppBlockPtr(new CppBlock()); \
        b->addLine("r.readPrimitive("+accessor+");"); \
        return b; \
    } \
    \
    CppBlockPtr upperType##Serializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const \
    { \
        CppBlockPtr b = CppBlockPtr(new CppBlock()); \
        b->addLine("if (not (" + accessor + " == " + otherInstanceAccessor + "))"); \
        b->addLine("\t return false;"); \
        return b; \
    }

    HANDLE_PRIMITIVE_CORRESPONDING
#undef RUN_ARON_MACRO
}
