/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <filesystem>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>
#include <RobotAPI/libraries/aron/core/codegenerator/ReaderInfo.h>
#include <RobotAPI/libraries/aron/core/codegenerator/WriterInfo.h>
#include <RobotAPI/libraries/aron/core/codegenerator/GenerateTypeInfo.h>

namespace armarx::aron::typereader
{
    template <typename Input>
    class Reader;

    template <typename Input>
    using ReaderPtr = std::shared_ptr<Reader<Input>>;

    template <typename Input>
    class Reader
    {
    public:
        Reader() = default;

        virtual void parseFile(const std::string& filename) = 0;
        virtual void parseFile(const std::filesystem::path& file) = 0;

        std::vector<std::string> getCodeIncludes() const
        {
            return codeIncludes;
        }
        std::vector<std::string> getAronIncludes() const
        {
            return aronIncludes;
        }
        std::vector<codegeneratorhelper::WriterInfoPtr> getWriters() const
        {
            return writers;
        }
        std::vector<codegeneratorhelper::ReaderInfoPtr> getReaders() const
        {
            return readers;
        }
        std::vector<codegeneratorhelper::GenerateObjectInfoPtr> getGenerateObjects() const
        {
            return generateObjects;
        }
        std::vector<codegeneratorhelper::GenerateIntEnumInfoPtr> getGenerateIntEnums() const
        {
            return generateIntEnums;
        }

    protected:
        std::vector<std::string> alreadyParsedXMLFiles;

        std::vector<std::string> codeIncludes;
        std::vector<std::string> aronIncludes;
        std::vector<codegeneratorhelper::ReaderInfoPtr> readers;
        std::vector<codegeneratorhelper::WriterInfoPtr> writers;
        std::vector<codegeneratorhelper::GenerateIntEnumInfoPtr> generateIntEnums;
        std::vector<codegeneratorhelper::GenerateObjectInfoPtr> generateObjects;
    };
}
