/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Simox
#include <SimoxUtility/json.h>

// Base Class
#include <RobotAPI/libraries/aron/core/io/dataIO/writer/WriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Config.h>

namespace armarx::aron::dataIO::writer
{
    class NlohmannJSONWriterToken;
    typedef std::shared_ptr<NlohmannJSONWriterToken> NlohmannJSONWriterTokenPtr;

    class NlohmannJSONWriterToken :
        virtual public WriterTokenInterface<nlohmann::json>
    {
    public:
        using PointerType = NlohmannJSONWriterTokenPtr;

    public:
        // constructor
        NlohmannJSONWriterToken() = delete;
        NlohmannJSONWriterToken(const data::Descriptor desc, const nlohmann::json& d)
        {
            descriptor = desc;
            element = d;
        }

        // public member functions
        virtual void addElement(const nlohmann::json& n) override
        {
            auto desc = this->getDescriptor();
            switch (desc)
            {
                case data::Descriptor::eDict:
                {
                    element[currentKey] = n;
                    break;
                }
                case data::Descriptor::eList:
                {
                    element.push_back(n);
                    currentIndex++;
                    break;
                }
                default:
                    throw error::DescriptorNotValidException("NlohmannJSONWriterToken", "addElement", "Could not resove a type of a token. Allowed are only containers.", desc);
            }
        }
    };
}
