/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD / STL
#include <memory>
#include <stack>
#include <sstream>

// Base Class
#include <RobotAPI/libraries/aron/core/io/typeIO/Writer.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Concepts.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/nlohmannJSON/NlohmannJSONWriterToken.h>

namespace armarx::aron::typeIO::writer
{
    class NlohmannJSONWriter :
        virtual public WriterInterface
    {
    public:
        NlohmannJSONWriter() = default;

        virtual void writeStartObject(const std::string&) override;
        virtual void writeEndObject() override;
        virtual void writeStartList() override;
        virtual void writeEndList() override;
        virtual void writeStartDict() override;
        virtual void writeEndDict() override;
        virtual void writeStartTuple() override;
        virtual void writeEndTuple() override;
        virtual void writeStartPair() override;
        virtual void writeEndPair() override;

        virtual void writeEigenMatrix(const std::vector<int>&, const std::string&) override;
        virtual void writeEigenQuaternion(const std::string&) override;
        virtual void writeIVTCByteImage(unsigned int w, unsigned int h, const std::string&) override;
        virtual void writeOpenCVMat(const std::vector<int>&, const std::string&) override;
        virtual void writePCLPointCloud(unsigned int w, unsigned int h, const std::string&) override;
        virtual void writePosition() override;
        virtual void writeOrientation() override;
        virtual void writePose() override;

        virtual void writeInt() override;
        virtual void writeLong() override;
        virtual void writeFloat() override;
        virtual void writeDouble() override;
        virtual void writeString() override;
        virtual void writeBool() override;
        virtual void writeTime() override;

        virtual void writeKey(const std::string&) override;

        nlohmann::json getResult() const
        {
            return lastRemovedToken->getElement();
        }

    private:
        bool wroteInitialStartObject = false;
        NlohmannJSONWriterTokenPtr lastRemovedToken = nullptr;
        std::stack<NlohmannJSONWriterTokenPtr> stack = {};

        Path generatePath();
    };
}
