/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// Simox
#include <SimoxUtility/json.h>

// Base Class
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/WriterToken.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Config.h>
#include <RobotAPI/libraries/aron/core/Descriptor.h>

namespace armarx::aron::typeIO::writer
{
    class NlohmannJSONWriterToken;
    typedef std::shared_ptr<NlohmannJSONWriterToken> NlohmannJSONWriterTokenPtr;

    class NlohmannJSONWriterToken :
        virtual public WriterTokenInterface<nlohmann::json>
    {
    public:
        using PointerType = NlohmannJSONWriterTokenPtr;

    public:
        // constructor
        NlohmannJSONWriterToken() = delete;
        NlohmannJSONWriterToken(const type::Descriptor desc, const nlohmann::json& t)
        {
            descriptor = desc;
            element = t;
        }

        // virtual member functions
        virtual void addElement(const nlohmann::json& n) override
        {
            auto desc = getDescriptor();
            switch (desc)
            {
                case type::Descriptor::eDict:
                {
                    element[io::Data::READER_WRITER_DICT_ACCEPTED_TYPE_SLUG] = n;
                    break;
                }
                case type::Descriptor::eList:
                {
                    element[io::Data::READER_WRITER_LIST_ACCEPTED_TYPE_SLUG] = n;
                    break;
                }
                case type::Descriptor::eObject:
                {
                    element[currentKey] = n;
                    break;
                }
                case type::Descriptor::eTuple:
                {
                    element.push_back(n);
                    break;
                }
                default:
                    throw error::DescriptorNotValidException("NlohmannJSONWriterToken", "addElement", "Could not resove a type of a navigator. Allowed are only containers.", desc);
            }
        }

        void setName(const std::string& n) override
        {
            auto desc = getDescriptor();
            if (desc != type::Descriptor::eObject)
            {
                throw error::DescriptorNotValidException("NlohmannJSONWriterToken", "setName", "Cant set the name of a non-object token.", desc);
            }
            element[io::Data::READER_WRITER_NAME_SLUG] = n;
        }
    };
}
